/*
  File Name  : ckEditorApi.js
  Description: CKEditor wrapper functions.

  Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability 
  to copy, modify, distribute) may be licensed in certain circumstances 
  (e.g., sample/template code intended to be used by customers to build upon and 
  distribute as their own products).

  DOORS - %full_filespec: ckeditorApi.js~15:javascript:ERS#1 %  %date_created: Tue Oct 23 15:14:26 2012 %

 (c) Copyright IBM Corporation 2011. All Rights Reserved.

  U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted
  by GSA ADP Schedule Contract with IBM Corp.
*/

// JavaScript API for DOORS client core code calls.

var lastToolbar = "";

// Keep latest editor contents, in case it needs to be restored after an invalid paste operation
var previousContent = ""

// Keep latest contents that was pasted
var contentsToPaste = ""

var dummyDivId = "dummyDiv"
var errorIcon = "/web/com.ibm.rdm.web/common/images/error.gif"

function removeIdAttributeFromPTags(html)
// DESCRIPTION: Remove all id attribute from all P tags found in html contents
// RETURNS: New contents with P tags without id attribute
{
    //evt.data.html = evt.data.html.replace(/<(P)[^>]+>/ig,'<$1>'); // This is good but all attributes are remove, only <p> is kept
    var wrapper = document.createElement('div');
    wrapper.innerHTML = html;
    var pElements = wrapper.getElementsByTagName('p');

    for (var i = 0; i < pElements.length; i++) 
    {
        pElements[i].removeAttribute('id');
        pElements[i].removeAttribute('ID');
    }

    var newHtml = wrapper.innerHTML;
    return newHtml;
}

//*********************************************************************************
// DESCRIPTION: KeyDown callback for CKEditor, 
//              this allows a selection of keydown events occuring within the CKEditor 
//              to be fired back to the Rich Client container window,
//              this gives the chance to the Rich Client to process specific shortcuts.
CKEDITOR.on('instanceCreated', function (e) {
    e.editor.on('contentDom', function () {
        var currentEditor = CKEDITOR.instances.editor1
        e.editor.document.on('click', function (e) 
        {
            if (currentEditor && currentEditor.name != dummyDivId)
            {
                var event = e.data.$;
                var linkTarget = e.data.getTarget();
                if (linkTarget.is('a') && linkTarget.$.href && (linkTarget.$.protocol.indexOf('http') == 0))
                {
                    if (event.ctrlKey == 1)
                    {
                        window.open(linkTarget.$.href);
                    }
                    currentEditor.document.$.body.contentEditable = true;
                }
                else if (linkTarget.is('img') && linkTarget.$.href && linkTarget.$.href.search(errorIcon) >= 0) 
                {
                    window.open(linkTarget.$.href);
                    e.data.preventDefault(); 
                }
            }
        });
    });
});

//*********************************************************************************
// DESCRIPTION: Gets the dialog definition, checks for link dialog 
//              removes target and advanced tabs for the link dialog
CKEDITOR.on('dialogDefinition', function (ev) {
    var dialogName = ev.data.name;
    var dialogDefinition = ev.data.definition;
    if (dialogName == 'link') {
        dialogDefinition.removeContents('target');
        dialogDefinition.removeContents('advanced');
    }
});

// Function splitting the supplied listString using a separator,
// and removing whitespaces from start and end of each entry 
// in the returned array.
function splitAndTrim(listString,separator)
{
    var firstResult = listString.split(separator);
    var trimmed = [];
    for (i in firstResult)
    {
        trimmed.push(CKEDITOR.tools.trim(firstResult[i])); 
    }
    return trimmed;
}

function restorePreviousContents()
// DESCRIPTION: Restore previous content to current editor.
{
    if (currentEditor)
    {
        setContents(previousContent);
    }
}

function getContentsToPaste()
// DESCRIPTION: Gets contents to paste.
// RETURNS: html contents.
{
    return contentsToPaste;
}
        
// Make the div contents editable by replacing it with an editor instance
// in the display.
function showEditor(toolbarString)
{
    var editor = CKEDITOR.instances.editor1;
    if (!editor)
    {
        lastToolbar = toolbarString;
        
        // Replace the <div id="editor1"> with an CKEditor
        // instance, using RRC style configurations
        var theToolbar = [];
        if (toolbarString.length != 0)
        {
            var toolbarStrings = splitAndTrim(toolbarString,":");
            for (i in toolbarStrings)
            {
                var barString = toolbarStrings[i];
                theToolbar.push(splitAndTrim(barString,","));
            }
        }
            
        editor = CKEDITOR.replace("editor1", {
            skin: (CKEDITOR.version == "3.4.2") ? "v2" : "rational",
        	toolbar: theToolbar,
        	height: '600px',
        	removePlugins: "elementspath,sametimeemoticons,customdialogs,lotustoolbars,urllink,lotusstatusmessage,lotusbidi,doclink",
        	coreStyles_bold: {
        		element: "b" // use a <b> instead of a <strong>
        	},
        	coreStyles_italic: {
        		element: "i" // use an <i> instead of an <em>
        	},
        	coreStyles_strike: {
        		element: "del" // use <del> instead of <strike>
        	},
        	enableTabKeyTools: true,
        	entities: false, // html entities are not valid xhml
        	entities_latin: false,
        	entities_greek: false
        });

        editor.on('instanceReady', function (e)
        {

            var editor = CKEDITOR.instances.editor1;
            editor.document.$.body.style.margin = "0px";

            // Resize the editor to match the browser window size.
            editor.resize(document.body.offsetWidth, document.body.offsetHeight);

            // Output filter rules..
            editor.dataProcessor.htmlFilter.addRules(
            {
                attributeNames:
                [
                // Discard the following attributes.
                    [(/^xml:lang$/), ''],
                    [(/^_fck_bookmark$/), ''],
                    [(/^_zipidx$/), ''],
                    [(/^unselectable$/), ''],
                    [(/^contentEditable$/), ''],
                ],

                elements:
                {
                    // Replace <font> with inert <span>
                    font: function (element)
                    {
                        element.name = 'span';
                        element.attributes = [];
                    },

                    // Discard any empty <tr> tags
                    tr: function (element)
                    {
                        if (element.children.length == 0)
                        {
                            return false;
                        }
                    }
                }
            });
        });

        editor.on('paste', function (evt)
        {
            var editor = evt.editor;
            previousContent = evt.editor.getData() // Keep previous contents before paste
            evt.data.html = removeIdAttributeFromPTags(evt.data.html); // Remove all id attributes for p tags
            contentsToPaste = evt.data.html; // Keep contents that was pasted
            setTimeout("window.location.replace('paste:')", 300); // Delegate validation of pasted contents to client container
        });        
    }    
}

// Hide the editor and display the read-only div.
function hideEditor(updateDiv)
{
    var editor = CKEDITOR.instances.editor1;
    if (editor)
    {
        // Initial call seems to fail first time for a given htmlEdit: 
        // try a 2nd time if so: seems to succeed.
        try
        {
            editor.destroy(!updateDiv);
        }
        catch(err)
        {
            try
            {
                editor.destroy(!updateDiv);
            }
            catch(err2)
            {
                // Do nothing.
            }
        }
    }
}

function getContents()
// DESCRIPTION: Gets the current contents of the editor.
// RETURNS: html contents.  Empty string on failure.
{
    var editor = CKEDITOR.instances.editor1;
    if (editor)
    {
        // Replace any &nbsp; instances: not valid xhtml
        return editor.getData().replace(/&nbsp;/g, "&#160;");
    }
    else
    {
        var theDiv = document.getElementById('editor1');
        if (!theDiv)
        {
            alert("getContents(): No editor!");
            return "";
        }
        else
        {
            return theDiv.innerHTML;
        }
    }
}

function getModified()
// DESCRIPTION: Has the editor contents been modified since loading?
// RETURNS: true iff it has been modified.
{
    var editor = CKEDITOR.instances.editor1;
    if (editor)
    {
        return editor.checkDirty();
    }
    else
    {
        return false;
    }
}

function resetModified()
// DESCRIPTION: Reset the modified flag of the editor to FALSE
// RETURNS: void
{
    var editor = CKEDITOR.instances.editor1;
    if (editor) {
        editor.resetDirty();
    }
}
/* end */