/*
  File Name  : gridEditorApi.js

  Description: CKEditor API functions for DOORS "HTML grid" control.

  Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability 
  to copy, modify, distribute) may be licensed in certain circumstances 
  (e.g., sample/template code intended to be used by customers to build upon and 
  distribute as their own products).

  DOORS - %full_filespec: gridCKEditorApi.js~20:javascript:ERS#1 %  %date_created: Tue Oct 23 15:13:52 2012 %

 (c) Copyright IBM Corporation 2012. All Rights Reserved.

  U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted
  by GSA ADP Schedule Contract with IBM Corp.

*/
    
var currentEditor = false;
var currentEditorId = "";

// welcomeContents is initialised the first time a new row is edited: 
//  the welcome text is a globalized text declared in core code (c++),
//  the core code will put the welcome text into a 'new row' contents cell by default,
//  then the javascript method used to enable an instance of ckeditor will be invoked, 
//  this is when we grab the value of the welcome text and initialise welcomeContents,
//  currently the logic to remove the welcome text and restore it if a new row contents is unchanged is done in the javascript.
var welcomeContents = ""; 

var newEditDivPrefix = "editDivNew_";   // copy of c++ newEditDivPrefix
var newRowVal = "new" ;                 // copy of c++ newRowVal
var welcomeRowVal = "welcome";          // copy of c++ welcomeRowVal
var editorDivClass = "editDiv";
var dummyDivId = "dummyDiv";
var toolbarOverlayId = "toolbarOverlay";
var errorIcon = "/web/com.ibm.rdm.web/common/images/error.gif"

// IMPORTANT: busyScripts is a counter used to protect ckeditor instance Creation/Destruction to happen at the same time,
// which will generate script errors, those errors are due to the async behaviour of ckeditor instanciation which uses timer():
//  - CKEDITOR.replace() called, which will trigger the following sequence of actions:
//      - instanceCreated() is invoked later by a timer
//      - then instanceReady() might be invoked later by a timer
//      - finally dataReady() might be invoked later by a timer
// At least ckeditor instance destruction is synchronous.
// If a new method makes access to the ckeditor instance, make sure you guard it as such:
//  function risky()
//  {
//      setScriptBusy();
//      .. do something with ckeditor
//      setScriptIdle();
//  }
var busyScripts = 0; // == 0 if nothing important is going on, will be increased by setScriptBusy(), and decreased by setSriptIdle()

// Keep latest editor contents, in case it needs to be restored after an invalid paste operation
var previousContent = ""

// Keep latest contents that was pasted
var contentsToPaste = ""

// Toolbar options
var toolbarString = "Format,Font,FontSize,-," +
"Bold,Italic,Underline,Strike,Subscript,Superscript,-," +
"Cut,Copy,Paste,PasteText,PasteFromWord,-,Undo,Redo,-,Find:" +
"TextColor,BGColor,-," +
"JustifyLeft,JustifyCenter,JustifyRight,JustifyBlock,Indent,Outdent,-," +
"BulletedList,NumberedList,Table,HorizontalRule,-," +
"Link,Unlink";
var theToolbar = [];

var verboseErrorHandling = false;

// http://dev.ckeditor.com/ticket/8480 workaround in WebBrowserEvents.cpp - symptom of fail is page reload
if (verboseErrorHandling) alert("LOAD");  

function removeIdAttributeFromPTags(html)
// DESCRIPTION: Remove all id attribute from all P tags found in html contents
// RETURNS: New contents with P tags without id attribute
{
    //evt.data.html = evt.data.html.replace(/<(P)[^>]+>/ig,'<$1>'); // This is good but all attributes are remove, only <p> is kept
    var wrapper = document.createElement('div');
    wrapper.innerHTML = html;
    var pElements = wrapper.getElementsByTagName('p');

    for (var i = 0; i < pElements.length; i++) 
    {
        pElements[i].removeAttribute('id');
        pElements[i].removeAttribute('ID');
    }

    var newHtml = wrapper.innerHTML;
    return newHtml;
}

function isScriptIdle()
// DESCRIPTION: Gets the idle state of the java script
// RETURNS: busyScripts, which is > 0 if the script is busy doing something important such as:
//          - ckeditor construction / destruction
{
    return (busyScripts == 0);
}

function setScriptBusy()
// DESCRIPTION: Increase busyScripts by one, this is a clear indication that the script is busy !
//              For safety with events, and to avoid script errors, all important methods should call setScriptBusy() on enter, 
//              and call setScriptIdle() on exit.
{
    busyScripts++;
}

function setScriptIdle()
// DESCRIPTION: Decrease busyScripts by one, if back to zero then this is a clear indication that the script is idle !
//              For safety with events, and to avoid script errors, all important methods should call setScriptBusy() on enter, 
//              and call setScriptIdle() on exit.
{
    if (busyScripts > 0)
    {
        busyScripts--;
    }
}

//*********************************************************************************
// DESCRIPTION: KeyDown callback for CKEditor, 
//              this allows a selection of keydown events occuring within the CKEditor 
//              to be fired back to the Rich Client container window,
//              this gives the chance to the Rich Client to process specific shortcuts.
CKEDITOR.on('instanceCreated', function (e)
{
    e.editor.on('contentDom', function ()
    {           
        e.editor.document.on('keydown', function (e)
        {    
            if (!isScriptIdle())
            {
                return;
            }
            
            currentEditor.document.$.body.contentEditable = true;
            if (currentEditor && currentEditor.name != dummyDivId)
            {                   
                var event = e.data.$;
                var keynum = event.keyCode;

                // Now stop the source event to propagate for specific key events;
                // this to stop IE default behavior (for instance ctrl+n, F5)
                if ((keynum == 116 /* F5 */) ||
                    (keynum == 27 /* Esc */) ||
                    (keynum == 9  /* Tab */) ||
                    (event.altKey == 1 && keynum == 38 /*Alt+Up arrow*/) ||
                    (event.altKey == 1 && keynum == 40 /*Alt+Down arrow*/) ||
                    (event.ctrlKey == 1 && keynum == 78 /*'n'*/) ||
                    (event.ctrlKey == 1 && keynum == 72 /*'h'*/) ||
                    (event.ctrlKey == 1 && keynum == 83 /*'s'*/)
                )
                {
                    // Display wait cursor because of timeouts below
                    currentEditor.document.$.body.style.cursor = 'wait';

                    // Re-fire the event to notify the Rich Client container of interesting key events;
                    // the event source must be changed to be currentEditorId so that the Rich Client process it.
                    // Note: Allow tab key events to fire: core code needs them to capture the focus target
                    if (keynum != 9)
                    {
                        // Cancel the default action
                        event.returnValue = false;
                        event.cancelBubble = true;
                        var theTarget = document.getElementById(currentEditor.name);
                        theTarget.fireEvent("onkeydown", event);
                    }

                    // Trigger core code action after this callback has returned - else can
                    // cause problems if the editor is replaced by the core code.
                    // Need a 300ms delay - still get problems at 200ms.
                    //
                    if (keynum == 27) // Escape key
                    {
                        setTimeout("window.location.replace('esc:" + keynum + "')", 300);
                    }
                    else if (keynum == 9) // Tab
                    {
                        // Send signal to core, to select captured focus target.
                        setTimeout("window.location.replace('tab:focus')", 300);
                    }
                    else if (event.altKey == 1)
                    {
                        //    *** Temporarily increasing Alt key navigation timeout value to minimise script errors
                        setTimeout("window.location.replace('alt:" + keynum + "')", 300); // ***
                    }
                    else
                    {
                        setTimeout("window.location.replace('ctrl:" + keynum + "')", 300);
                    }
                }

            }
        });

        //*********************************************************************************
        // DESCRIPTION: Gets the dialog definition, checks for link dialog 
        //              removes target and advanced tabs for the link dialog
        CKEDITOR.on('dialogDefinition', function (ev) {
            var dialogName = ev.data.name;
            var dialogDefinition = ev.data.definition;
            if (dialogName == 'link') {
                dialogDefinition.removeContents('target');
                dialogDefinition.removeContents('advanced');
            }
        });

        e.editor.document.on('click', function (e)
        {
            if (!isScriptIdle())
            {
                return;
            }
            
            if (currentEditor && currentEditor.name != dummyDivId)
            {
                var event = e.data.$;
                var linkTarget = e.data.getTarget();
                if (linkTarget.is('a') && linkTarget.$.href && (linkTarget.$.protocol.indexOf('http') == 0))
                {
                    if (event.ctrlKey == 1)
                    {
                        window.open(linkTarget.$.href);
                    }
                    currentEditor.document.$.body.contentEditable = true;
                }
                else if (linkTarget.is('img') && linkTarget.$.href && linkTarget.$.href.search(errorIcon) >= 0) 
                {
                    window.open(linkTarget.$.href);
                    e.data.preventDefault();
                }
            }
        });
    });
    
    // Ckeditor is not activated on a dummy div, therefore instanceReady() and dataReady() will not be called,
    // release the idle state here.
    if (currentEditorId == dummyDivId)
    {
        setScriptIdle();
    }
});

function getContents()
// DESCRIPTION: Gets the current contents of the current editor or newEditDivId.
// RETURNS: html contents.  Empty string on failure.
{    
    var contents = "";
    if (currentEditor)
    {
        // Replace any &nbsp; instances: not valid xhtml
        contents = currentEditor.getData().replace(/&nbsp;/g, "&#160;");       
    }
    else
    {
        var theDiv = document.getElementById(currentEditorId);
        
        if (!theDiv)
        {
            alert("getContents(): No editor! " + currentEditorId);
        }
        else
        {
            contents = theDiv.innerHTML;
        }
    }

    return contents;
}

function restorePreviousContents()
// DESCRIPTION: Restore previous content to current editor.
{
    if (currentEditor)
    {
        setContents(previousContent);
    }
}

function getContentsToPaste()
// DESCRIPTION: Gets contents to paste.
// RETURNS: html contents.
{
    return contentsToPaste;
}

function setContents(contents)
// DESCRIPTION: Sets the current contents of the current editor or newEditDivId.
{  
    setScriptBusy();    
    if (currentEditor)
    {
        // Replace any &nbsp; instances: not valid xhtml
        currentEditor.setData(contents);
    }
    else
    {
        var theDiv = document.getElementById(currentEditorId);
            
        if (!theDiv)
        {
            alert("getContents(): No editor! " + currentEditorId);
        }
        else
        {
            theDiv.innerHTML = contents;
        }
    }
    setScriptIdle();
}

function getModified()
// DESCRIPTION: Checks whether the contents of the current editor 
//              have been modified by the user.
// RETURNS: true if the editor exists and has been edited, else false.
{            
    setScriptBusy();  
        
    // Exit if the dummy div contains the current editor,
    // the dummy div can't be edited !  
    var modified = currentEditor 
        && currentEditor.name != dummyDivId
        && currentEditor.isReady
        && currentEditor.checkDirty();
    
    setScriptIdle();
    
    return modified;
}

function splitAndTrim(listString,separator)
// DESCRIPTION: Function splitting the supplied listString using a separator,
//                 and removing whitespaces from start and end of each entry 
//                 in the returned array.
//                 Used in showEditor to build toolbar from input string.
// RETURNS: An array of strings  
{
    var firstResult = listString.split(separator);
    var trimmed = [];
    
    for (i in firstResult)
    {
        trimmed.push(CKEDITOR.tools.trim(firstResult[i])); 
    }
    
    return trimmed;
}

function enableToolbar()
// DESCRIPTION: Enables the CKEditor toolbar by hiding the masking overlay.
{
    window.document.getElementById(toolbarOverlayId).style.visibility = "hidden";
}

function disableToolbar()
// DESCRIPTION: Disables the CKEditor toolbar by showing the masking overlay.
{
    window.document.getElementById(toolbarOverlayId).style.visibility = "visible";
}

function hideEditor(updateDiv)
// DESCRIPTION: Hide the current editor, 
//              and restore the container div with maybe some new text.
//        NOTE: This is also called from cpp
{  
    setScriptBusy();     
    
    if (currentEditor)
    {           
        if (arguments.length == 0)  // cpp specifies an argument
        {
            updateDiv = (currentEditor.name != dummyDivId);   // default to no update on dummyDiv
        }
        
        currentEditor.destroy(!updateDiv);
        currentEditor = false;
        
        // IMPORTANT:
        //  This code is responsible for restoring the welcome text for un-changed new row contents,
        //  in other words, if the row is new and the user has not done any changes after editing,
        //  then at destruction of the ckeditor instance the welcome text must be restored to the new cell div,
        //  the welcome text is currently kept in a variable called welcomeContents.
        var cellEditDiv = window.document.getElementById(currentEditorId);
        var parentRow = cellEditDiv.parentNode.parentNode;
       
        if (parentRow.getAttribute('state') == newRowVal
             || parentRow.getAttribute('state') == welcomeRowVal) // "new" or "welcome"
        {
            if (getContents() == "") // Still empty (in ckeditor term) ?
            {
                cellEditDiv.innerHTML = welcomeContents; // Restore the welcome text
            }
        }
        //
        
        currentEditorId = "";
        disableToolbar();       
    }
    
    setScriptIdle();
}

function prepareBeforeShowingEditor(divId)
// DESCRIPTION: Remove previous editor, prepare transition of next editor to a new container div.
// RETURNS: Cell id of the next editor container.
{
    setScriptBusy();
    var theDiv = false;
    if (divId != dummyDivId)
    {
        // Find a well known (div) element that can contain the editor
        var divStart = window.document.getElementById(divId);
        for (theDiv = divStart; theDiv && (!theDiv.className || theDiv.className.indexOf(editorDivClass) < 0); theDiv = theDiv.parentNode) { }
        if (!theDiv && verboseErrorHandling)    // this is an error - the effect of this will be a failed start of the editor
        {
            var msg = "show(" + divId + ") Failed to find an element with class containing '" + editorDivClass + "' from:\n\t";
            for (theDiv = divStart; theDiv && (!theDiv.className || theDiv.className.indexOf(editorDivClass) < 0); theDiv = theDiv.parentNode)
            {
                msg += theDiv.id + "/" + theDiv.className + ";  "; theDiv = theDiv.parentNode;
            }
            alert(msg);
        }
    }

    if (theDiv && theDiv.id != "")        // A container div has been found
    {
        setScriptIdle();
        hideEditor();
        return theDiv.id;
    }

    // Either want the dummyDiv, or no container found (an error condition),
    // so return dummy div to keep toolbar visible. 
    if (currentEditor && currentEditor.name == dummyDivId)
    {
        setScriptIdle();
        return "";  // Already in dummy div, no change needed
    }

    // Active editor in dummy div
    setScriptIdle();
    hideEditor();
    return dummyDivId;
}

function showEditor(sourceDivId)
// DESCRIPTION: Show the CKEditor
//     RETURNS: currentEditor
//        NOTE: This is also called from cpp
{   
    if (!isScriptIdle())
    {
        return;
    }
    // Turn off window event driven resizes whilst we jigger about the display (hide/re-show toolbar)
    enableAutoScrollResize = false;

    // Get the container id
    var nextEditorId = prepareBeforeShowingEditor(sourceDivId);

    if (nextEditorId == "")
    {
        // No valid container has been found, or already in desired div
        resizeScrollDiv();  // (also turns back on window event driven resizes)
        return currentEditor;
    }
 
    var isDummyDiv = (nextEditorId == dummyDivId);
    if (isDummyDiv)
    {
        disableToolbar();
    }
    else
    {
        enableToolbar();
    }

    // Replace the <div id=nextEditorId> with a CKEditor
    // instance, using RRC style configurations
    var editor = CKEDITOR.instances.nextEditorId;
    currentEditorId = nextEditorId;
    currentEditor = editor; // may be null

    var theDiv    = window.document.getElementById(nextEditorId);
    var theRow    = theDiv.parentNode.parentNode;
    var isHeading = theRow.getAttribute('heading') === "1";

    // IMPORTANT:
    //  This code is reponsible for removing the welcome text of a new row before activating the ckeditor,
    //  this to avoid showing the welcome text while editing,
    //  welcomeContents will be initialised here to whatever welcome text is present in the first editable new row contents cell. 
    if (nextEditorId.indexOf(newEditDivPrefix) == 0 // "editDivNew_*"
        && (theRow.getAttribute('state') == newRowVal || theRow.getAttribute('state') == welcomeRowVal)) // "new" or "welcome"
    {
        // If it has a genuine binding ID, then it could be an artifact re-use
        var bindingId = theRow.getAttribute('dataBindingId');
        if (bindingId.length == 0 || bindingId.indexOf(newEditDivPrefix) == 0)
        {
            // First time called?
            // then assume that the contents is a welcome text and save it to welcomeContents
            if (welcomeContents == "") 
            { 
                welcomeContents = theDiv.innerHTML;
            }

            // Currently showing the welcome text?
            if (welcomeContents != "" && getContents() == welcomeContents)
            {         
                theDiv.innerHTML = ""; // Replace the welcome text by a genuinely empty text (not an &nbsp; that may be appended to on edit)
            }
        }
    }
    //

    if (!editor)
    {
        // Set the height of the editing area
        //CKEDITOR.config.height = theDiv.offsetHeight;
        //CKEDITOR.config.resize_enabled = true;

        // Commented interesting setting for future use
        //CKEDITOR.config.scayt_autoStartup = false;
        //CKEDITOR.config.startupShowBorders = false;
        //CKEDITOR.config.startupOutlineBlocks = false;
        //CKEDITOR.config.resize_minWidth = 100;
        //CKEDITOR.config.resize_minHeight = 100;

        // Build the toolbar, once only
        if (theToolbar.length == 0 && toolbarString.length != 0)
        {
            var toolbarStrings = splitAndTrim(toolbarString, ":");
            for (i in toolbarStrings)
            {
                var barString = toolbarStrings[i];
                theToolbar.push(splitAndTrim(barString, ","));
            }
        }
        
        setScriptBusy();
         
        currentEditor = editor = CKEDITOR.replace(nextEditorId, 
        {
            skin: (CKEDITOR.version == "3.4.2") ? "v2" : "rational",
            toolbar: theToolbar,
            sharedSpaces: { top: 'sharedToolbar' },
            height: theDiv.offsetHeight,
            width: theDiv.offsetWidth,

            // Disable autoGrow on the dummyDiv - avoids a couple of timing dependant script errors on ckeditor resizing
            autoGrow_onStartup: !isDummyDiv,
            autoGrow_minHeight: 15,
            //autoGrow_bottomSpace: -20,
            autoParagraph: false,
            removePlugins: "resize,maximise,elementspath,sametimeemoticons,lotustoolbars,urllink,lotusstatusmessage,lotusbidi,doclink" + (isDummyDiv ? ",autogrow" : ""),
            extraPluginList: (isDummyDiv ? "": "autogrow"),
            format_tags: "p;h1;h2;h3;h4;h5;h6", // Skip the Formatted, Address, Normal (DIV) options from the Format menu

            coreStyles_bold:   { element: "b" },    // use a <b> instead of a <strong>
            coreStyles_italic: { element: "i" },    // use an <i> instead of an <em>
            coreStyles_strike: { element: "del" },  // use <del> instead of <strike>

            enableTabKeyTools: true,
            entities: false,
            entities_latin: false,
            entities_greek: false
        });

        editor.isReady = false;

        editor.on('instanceReady', function (e)
        {
            try
            {
                var editor = CKEDITOR.instances[nextEditorId];
                if (editor == null)     // It is possible for an editor to be destroyed before its instanceReady has fired.
                {                       // Exit immediately if so to minimise scripting errors (there will likely still be some).
                    if (verboseErrorHandling)
                    {
                        alert("ERROR: instanceReady on zombie ckeditor - " + this.name + " already destroyed.");
                    }

                    return;
                }
                //alert("instanceReady " + this.name + "\n" + this.getData());

                // Make sure the vertical scrollbar is not displayed
                editor.document.$.documentElement.style.overflowY = "hidden";
                editor.document.$.body.style.overflowY = "hidden";
                editor.document.$.body.style.overflowX = "hidden";
                editor.document.$.body.style.wordWrap = "break-word";
                editor.document.$.body.style.margin = "0px";
                editor.document.$.body.style.padding = "0px";
                
                if (editor.name == dummyDivId)
                {
                    enableAutoScrollResize = true;    // not strictly required, resizeScrollDiv() does it anyway
                    resizeScrollDiv();
                    window.document.getElementById(dummyDivId).style.display = "none";
                    setScriptIdle();
                    return;
                }

                if (isHeading)
                {
                    editor.document.$.body.style.fontSize = "22px";
                    editor.document.$.body.style.fontWeight = "bold";
                }
                else
                {
                    editor.document.$.body.style.fontSize = "12px";
                }

                // Ensure the scrolling div accommodates the resized editor
                // along with the rest of the grid.
                resizeScrollDiv();

                // Output filter rules for editor.getData()
                editor.dataProcessor.htmlFilter.addRules(
                {
                    attributeNames:
                    [
                    // Discard the following attributes.
                        [(/^xml:lang$/), ''],
                        [(/^_fck_bookmark$/), ''],
                        [(/^_zipidx$/), ''],
                        [(/^unselectable$/), ''],
                        [(/^contentEditable$/), ''],
                    ],

                    elements:
                    {
                        // Replace <font> with inert <span>
                        font: function (element)
                        {
                            element.name = 'span';
                            element.attributes = [];
                        },

                        // Discard any empty <tr> tags
                        tr: function (element)
                        {
                            if (element.children.length == 0)
                            {
                                return false;
                            }
                        }
                    }
                }); 
            }
            catch (err)
            {
                if (verboseErrorHandling)
                {
                    alert("ERROR: ckeditor instanceReady exception " + editor.name + '\n' + err.name + '\n' + err.message);
                }

                return;
            }
            
        });

        editor.on('dataReady', function (e)
        {
            if (CKEDITOR.instances[this.name] != null)   // In case already destroyed (zombie ckeditor)
            {
                if (this.name != dummyDivId)                 // Never give focus to dummyDiv - it can't be edited
                {
                    var theDiv = window.document.getElementById(this.name);
                    var yPos = findPos(theDiv);
                    window.scroll(0, yPos);
                    this.focus();
                    //alert("dataReady " + editor.name + " yPos=" + yPos + "\n" + this.getData().replace(/\t/g, "\\t").replace(/\n/g, "\\n"));
                }
                this.isReady = true;
            }  

            // Release the idle state here, don't realease on instanceReady, because this is too early
            setScriptIdle();
        });
        
        editor.on('destroy', function (e)
        {
        });

        editor.on('paste', function (evt)
        {
            var editor = evt.editor;
            previousContent = evt.editor.getData(); // Keep previous contents before paste
            evt.data.html = removeIdAttributeFromPTags(evt.data.html); // Remove all id attributes for p tags
            contentsToPaste = evt.data.html; // Keep contents that was pasted         
            setTimeout("window.location.replace('paste:')", 300); // Delegate validation of pasted contents to client container
        });        
    }
    else
    {
        enableAutoScrollResize = true;    // restore window event driven resizes 
    }
    
    return editor;
}

function showToolbar()
// DESCRIPTION: Show the CKEditor toolbar, by associating it with the dummy Div.
{
    showEditor(dummyDivId);
} 

function getEditorId()
// DESCRIPTION: Returns the current div ID the editor is created on
//        NOTE: This is more the "most recently instantiated", which 
//              may not be fully constructed yet.
{
    return currentEditorId;
}

/* End */